<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Services\PaypalService;
use Carbon\Carbon;
use App\OrdenDetalle;
use App\User;
use App\Role;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use App\Mail\OrdenUsuarioRegistrado;
use App\Jobs\SendEmailOrdenUsuario;
use App\Jobs\SendOrdenCreacionJob;
use App\Jobs\SendOrdenCreacionProductoJob;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\DB;
use App\Configuracion;
use App\ProductoAlmacen;
use App\Order;
use App\Producto;

class PaymentController extends Controller
{
    public function pay(Request $request){

        session()->put('productos', $request->all());

        $rules = [
            'value' => ['required', 'numeric', 'min:5'],
            'currency' => ['required', 'exists:monedas,iso'],
            'payment_platform' => ['required', 'exists:plataforma_pagos,id'],
        ];

        $request->validate($rules);

        $paymentPlatform = resolve(PaypalService::class);

        return $paymentPlatform->handlePayment($request);
    }

    public function approval(){

        $paymentPlatform = resolve(PaypalService::class);

        $payment = $paymentPlatform->handleApproval();

        return $this->orderCreate($payment);
    }

    public function cancelled(){
        
        return redirect()->route('home.productos')->withErrors('Pago cancelado');
    }

    public function orderCreate($payment){

        $user = $payment->payer;
        $data = session()->get('productos');
        
        if(User::where('email',$user->email_address)->exists()){
            DB::beginTransaction();
            try {

                $this->createOrden($data, $payment);

                DB::commit();
            }catch (Exception $e) {
                DB::rollBack();

                return [
                    'success' => false,
                    'message' => $e->getMessage()
                ];
            }
        }else{
            DB::beginTransaction();
            try {

                $this->RegistroPagoRapido($user);
                $this->createOrden($data, $payment);

                DB::commit();
            }catch (Exception $e) {
                DB::rollBack();

                return [
                    'success' => false,
                    'message' => $e->getMessage()
                ];
            }
        }

        $payment_price = $payment->purchase_units[0]->payments->captures[0]->amount;

        return redirect()
                ->route('home.productos')
                ->withStatus("Gracias, {$user->name->given_name}. Tu Orden por el monto de {$payment_price->value}{$payment_price->currency_code} fue pagado con exito.");
    }

    public function createOrden($request, $payment)
    {
        $productos = $request['productos'];
        $transaction_id = $payment->id;
        $fraud_score = 0;
        $total = $payment->purchase_units[0]->payments->captures[0]->amount->value;
        $num_orden = $request['descripcion'];
        $email = $payment->payer->email_address;

        $now = Carbon::now();

        $orden = new Order;
        if(Auth::check()){
            $orden->user_id = Auth::user()->id;
        }else{
            $usuario = User::where('email', $email)->first();
            $orden->user_id = $usuario->id;
        }
        $orden->transaction_id = $transaction_id;
        $orden->num_orden = $num_orden;
        $orden->total = $total;

        if($fraud_score >= 15){
            $orden->revision_manual = true;
            $orden->status = 'Detenida';
        }else{
            $orden->status = 'Pendiente';
            $orden->revision_manual = false;
        }
        $orden->fecha_creacion = $now->format('Y-m-d H:i');
        $orden->fecha_limite = $now->format('Y-m-d H:i');
        $orden->fecha_entrega = $now->addDay(5)->format('Y-m-d H:i');
        $orden->comercio_id = $productos[0]['comercio_id'];

        $orden->save();

        foreach($productos as $producto){

            $id = intval($producto['id']);
            $cantidad = intval($producto['cantidad']);
            $precio = floatval($producto['precio'])* floatval($producto['cantidad']);
            
            $this->saveOrderDetail($id, $cantidad, $precio, $orden->id);
            $this->decreaseQuantities($id, $cantidad);
        }

        dispatch(new SendOrdenCreacionJob($orden, $productos));

        return $orden;
    }


    protected function decreaseQuantities($id , $cantidad)
    {
        $product = Producto::findOrFail(intval($id));

        $configuracion = Configuracion::where('comercio_id', $product->comercio_id)->first();

        if($configuracion->almacen_id){
            $producto_almacen = ProductoAlmacen::where('almacen_id', $configuracion->almacen_id)->first();
            $producto_almacen->update(['stock' => $producto_almacen->stock - intval($cantidad)]);
        }
    }

    protected function incrementQuantities($id, $cantidad)
    {
        $product = Producto::findOrFail($id);

        $configuracion = Configuracion::where('comercio_id', $product->comercio_id)->first();

        if($configuracion->almacen_id){
            $producto_almacen = ProductoAlmacen::where('almacen_id', $configuracion->almacen_id)->first();
            $producto_almacen->update(['stock' => $producto_almacen->stock - intval($cantidad)]);
        }
    }

    protected function saveOrderDetail($id, $cantidad, $precio, $order_id){

        OrdenDetalle::create([
            'orden_id'      => $order_id,
            'producto_id'   => $id,
            'cantidad'      => $cantidad,
            'precio'        => $precio,
        ]);
    }

    public function RegistroPagoRapido($user) {

        //Generando Password aleatorio
        $pwd = str_random(12);

        $token = $this->createNewToken();

        $user = User::create([
            'cl_nombre' => $user->name->given_name,
            'cl_documento' => '00000000',
            'cl_celular' => '+51 000 000 000',
            'email' => $user->email_address,
            'password' => Hash::make($pwd),
            'token' => $token,
            'status' => true,
        ]);

        $user
            ->roles()
            ->attach(Role::where('name', 'user')->first());

        //Envio de Email
        
        dispatch(new SendEmailOrdenUsuario($user->email_address, $pwd, $token));

    }

    public function createNewToken()
    {
        return hash_hmac('sha256', Str::random(40), env('APP_KEY'));
    }
}
