<?php

namespace App\Services;

use Illuminate\Http\Request;
use App\Traits\ConsumeExternalServices;

class PaypalService
{
    use ConsumeExternalServices;

    protected $baseUri;

    protected $clientId;

    protected $clientSecret;

    public function __construct()
    {
        $this->baseUri = config('services.paypal.base_uri');
        $this->clientId = config('services.paypal.client_id');
        $this->clientSecret = config('services.paypal.client_secret');
    }

    public function resolveAuthorization(&$queryParams, &$formParams, &$headers)
    {
        $headers['Authorization'] = $this->resolveAccessToken();
    }

    public function decodeResponse($response)
    {
        return json_decode($response);
    }

    public function resolveAccessToken()
    {
        $credentials = base64_encode("{$this->clientId}:{$this->clientSecret}");

        return "Basic {$credentials}";
    }

    public function handlePayment(Request $request)
    {
        $order = $this->createOrder($request->value, $request->currency);

        $orderLinks = collect($order->links);

        $approve = $orderLinks->where('rel', 'approve')->first();

        session()->put('approvalId', $order->id);

        return [
            'success' => true,
            'link' => $approve->href,
            'message' => 'Sera redirigido a PayPal'
        ];
    }

    public function handleApproval()
    {
        if(session()->has('approvalId')){
            $approvalId = session()->get('approvalId');

            $payment = $this->capturePayment($approvalId);

            $name = $payment->payer->name->given_name;
            $payment_price = $payment->purchase_units[0]->payments->captures[0]->amount;
            $amount = $payment_price->value;
            $currency = $payment_price->currency_code;
            
            /* return redirect()
                    ->route('home.productos')
                    ->withStatus("Gracias, {$name}. Tu Orden por el monto de {$amount}{$currency} fue pagado con exito."); */

            return $payment;
        }

        return redirect()->route('home.productos')->withErrors('No se puede capturar el pago. Intente nuevamente.');
    }

    public function createOrder($value, $currency)
    {
        return $this->makeRequest(
            'POST',
            'v2/checkout/orders',
            [],
            [
                'intent' => 'CAPTURE',
                'purchase_units' => [
                    0 => [
                        'amount' => [
                            'currency_code' => strtoupper($currency),
                            'value' => $value,
                        ]
                    ]
                ],
                'application_context' => [
                    'brand_name' => config('app.name'),
                    'shipping_preference' => 'NO_SHIPPING',
                    'user_action' => 'PAY_NOW',
                    'return_url' => route('paypal.approval'),
                    'cancel_url' => route('paypal.cancelled'),
                ]
            ],
            [],
            $isJsonRequest = true
        );
    }

    public function capturePayment($approvalId)
    {
        return $this->makeRequest(
            'POST',
            "v2/checkout/orders/{$approvalId}/capture",
            [],
            [],
            [
                'Content-Type' => 'application/json',
            ]
        );
    }
}